@@ -34,6 +34,7 @@
 
 #include "pgext.h"
 #include "hsvlrgb.h"
+#include "getset.h"
 
 /*
     Set all opaque (alpha>0) pixels to specified color (R,G,B).
@@ -44,10 +45,13 @@ setColor( PyObject* self, PyObject* args )
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
     int w,h,x,y = 0;
+    char blend = 0;
     SDL_PixelFormat *format = NULL;
     PyObject *py_color = NULL;
     Uint8 color[4];
-    Uint8 r,g,b,a;
+    Uint8 r=0,g=0,b=0,a=0;
+    float sa=0, da=0, oa=0;
+    Uint8 sr=0, sg=0, sb=0;
 
     PY_PARSEARGS_BEGIN
         "OO", &py_surface, &py_color
@@ -59,11 +63,33 @@ setColor( PyObject* self, PyObject* args )
         Py_INCREF(Py_None);return Py_None;
     }
 
+    if(color[3] == 0){
+        Py_INCREF(Py_None);return Py_None;
+    }
+
+    if(color[3] != 255){
+        blend = 1;
+        sa = (float) (((float) color[3]) / 255.0f);
+        sr = (Uint8) (((float) color[0]) * sa);
+        sg = (Uint8) (((float) color[1]) * sa);
+        sb = (Uint8) (((float) color[2]) * sa);
+    }
+
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
             getPixel_RGBA(surface, x, y, &r, &g, &b, &a);
             if(a==0) continue;
-            setPixel_RGBA(surface, x, y, color[0], color[1], color[2], a );
+            if(blend){
+                da = (float) (((float) a) / 255.0f);
+                da = da * (1 - sa);
+                oa = sa + da;
+                r = clampUint8((((float)r * da) + sr) / oa );
+                g = clampUint8((((float)g * da) + sg) / oa );
+                b = clampUint8((((float)b * da) + sb) / oa );
+                setPixel_RGBA(surface, x, y, r, g, b, a );
+            } else {
+                setPixel_RGBA(surface, x, y, color[0], color[1], color[2], a );
+            }
         }
     }
 
@@ -99,21 +125,20 @@ setAlpha( PyObject* self, PyObject* args )
         shift = clampUint8( shift );
     }
     if( method == 2 ){
-        percent = ((double)(shift))/100.0;
+        percent = ((double)(shift)) / 100.0f;
     }
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
             getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
             if(method == 0){
-                a = a + shift;
-                a = clampUint8( a );
+                a = clampUint8(a + shift);
             }
             if(method == 1){
                 a = shift;
             }
             if(method == 2){
-                a = a * percent;
+                a = (Uint8) (a * percent);
             }
             setPixel_RGBA(surface, x, y, r, g, b, a );
         }
@@ -149,27 +174,61 @@ greyscale( PyObject* self, PyObject* args )
 
     getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
 
-    for(x = 0; x < w; x ++){
-        for(y = 0; y < h; y ++){
-            getPixel_RGBA(surface, x, y, &r, &g, &b, &a);
-            if(method == 0){
-                v = (r+g+b)/3.0;
-                setPixel_RGBA(surface, x, y, v, v, v, a);
-            } else if(method == 1){
-                RGB2HSV( r, g, b, &hue, &sat, &val );
-                HSV2RGB( hue, 0, val, &r, &g, &b );
-                setPixel_RGBA(surface, x, y, r, g, b, a);
-            } else if(method == 2){
-                RGB2HSL( r, g, b, &hue, &sat, &val );
-                HSL2RGB( hue, 0, val, &r, &g, &b );
-                setPixel_RGBA(surface, x, y, r, g, b, a);
-            }
-        }
+    if(method == 0){
+        GETPIXELS_RGBA(surface);
+            v = (int) ((float) (r + g + b) / 3.0f);
+            setPixel_RGBA(surface, x, y, v, v, v, a);
+        }}
+    } else if(method == 1){
+        GETPIXELS_RGBA(surface);
+            RGB2HSV( r, g, b, &hue, &sat, &val );
+            HSV2RGB( hue, 0, val, &r, &g, &b );
+        SETPIXELS_RGBA(surface, a);
+    } else if(method == 2){
+        GETPIXELS_RGBA(surface);
+            RGB2HSL( r, g, b, &hue, &sat, &val );
+            HSL2RGB( hue, 0, val, &r, &g, &b );
+        SETPIXELS_RGBA(surface, a);
     }
-
     Py_INCREF(Py_None);return Py_None;
 }
 
+/*
+    Desaturate
+*/
+static PyObject*
+desaturate( PyObject* self, PyObject* args )
+{
+    PyObject* py_surface = NULL;
+    SDL_Surface *surface;
+    int w,h,pxcnt = 0;
+    SDL_PixelFormat *format = NULL;
+    Uint8 r,g,b,a;
+    int eq = 0;
+    float percent = 0;
+
+    PY_PARSEARGS_BEGIN
+        "Of", &py_surface, &percent
+    PY_PARSEARGS_END
+
+    getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
+    pxcnt = w * h;
+    if(percent >= 1.0){
+        Py_INCREF(Py_None);return Py_None;
+    }
+
+    while( pxcnt-- ){
+        getPixelN_RGBA(surface, pxcnt, &r, &g, &b, &a );
+        eq = (int) ((float) (r + g + b) / 3.0f);
+        r = (Uint8)(r + (((float) (eq - r)) * percent));
+        g = (Uint8)(g + (((float) (eq - g)) * percent));
+        b = (Uint8)(b + (((float) (eq - b)) * percent));
+        setPixelN_RGBA(surface, pxcnt, r, g, b, a );
+    }
+    Py_INCREF(Py_None);return Py_None;
+}
+
+
 
 /*
     Invert surface colors
@@ -217,7 +276,7 @@ contrast( PyObject* self, PyObject* args )
 
     getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
-    
+
     factor = (259 * ((float)cvalue + 255)) / (255 * (259 - cvalue));
 
     while( pxcnt-- ){
@@ -239,7 +298,6 @@ brightness( PyObject* self, PyObject* args )
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
     int w,h,pxcnt = 0,bvalue = 0;
-    float factor = 0;
     SDL_PixelFormat *format = NULL;
     Uint8 r,g,b,a;
 
@@ -249,7 +307,7 @@ brightness( PyObject* self, PyObject* args )
 
     getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
-    
+
     while( pxcnt-- ){
         getPixelN_RGBA(surface, pxcnt, &r, &g, &b, &a);
         r = clampUint8(r + bvalue);
@@ -469,8 +527,8 @@ multiply( PyObject* self, PyObject* args )
     SDL_Surface *surface;
     int w,h,pxcnt = 0;
     SDL_PixelFormat *format = NULL;
-    float shift = 1.0;
-    float rp = 1.0, gp = 1.0, bp = 1.0;
+    float shift = 1.0f;
+    float rp = 1.0f, gp = 1.0f, bp = 1.0f;
     char rb = 1, gb = 1, bb = 1;
     Uint8 r,g,b,a;
 
@@ -508,7 +566,6 @@ colorize( PyObject* self, PyObject* args )
     PyObject* py_surface = NULL;
     SDL_Surface *surface;
     int w,h,x,y = 0;
-    Uint32 pixel;
     SDL_PixelFormat *format = NULL;
     int check_hue,new_hue;
     int s_shift = 0;
@@ -525,32 +582,25 @@ colorize( PyObject* self, PyObject* args )
 
     getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
 
-    for( y = 0; y < h; y ++ ){
-        for( x = 0; x < w; x ++ ){
-            pixel = getPixel(surface, x, y);
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
-            RGB2HSV( r,g,b, &hue, &sat, &val );
-            if( (int)hue == check_hue ){
-                hue = new_hue;
-                if( s_shift != 0 || v_shift != 0 ){
-                    sq = sat/100.f;
-                    if( s_shift != 0 ){
-                        sat += ( ((double)(s_shift)) *sq);
-                        sat = clampValue( sat, 0.0, 100.0 );
-                    }
-                    if( v_shift != 0 ){
-                        val += ( ((double)(v_shift)) *sq);
-                        val = clampValue( val, 0.0, 100.0 );
-                    }
+    GETPIXELS_RGBA(surface);
+        RGB2HSV( r,g,b, &hue, &sat, &val );
+        if( (int)hue == check_hue ){
+            hue = new_hue;
+            if( s_shift != 0 || v_shift != 0 ){
+                sq = (float) sat / 100.0f;
+                if( s_shift != 0 ){
+                    sat += ( ((double)(s_shift)) *sq);
+                    sat = clampValue( sat, 0.0f, 100.0f );
+                }
+                if( v_shift != 0 ){
+                    val += ( ((double)(v_shift)) *sq);
+                    val = clampValue( val, 0.0f, 100.0f );
                 }
-                count++;
             }
-            HSV2RGB( hue,sat,val, &r, &g, &b );
-            pixel = SDL_MapRGBA( format, r, g, b, a );
-            setPixel(surface, x, y, pixel);
-
+            count++;
         }
-    }
+        HSV2RGB( hue,sat,val, &r, &g, &b );
+    SETPIXELS_RGBA(surface, a);
     return Py_BuildValue("i", count );
 }
 
@@ -584,49 +634,43 @@ alphaMask( PyObject* self, PyObject* args )
     getSurfaceAttrs(py_surface, &w, &h, &format, &surface);
     target_surface = createSurface( mw, mh );
 
-
-    /*if(format->BytesPerPixel == 3){
-        return RAISE (PyExc_ValueError, "input surface must be 32bit surface not 24bit");
-    }*/
-
     if( mw != w || mh != h ){
         return RAISE (PyExc_ValueError, "mask must be same size as surface");
     }
 
-    for( y = 0; y < h; y ++ ){
-        for( x = 0; x < w; x ++ ){
-            /* get mask pixel*/
-            getPixel_RGBA(surface_mask, x, y, &r, &g, &b, &a );
-
-            if( method == 0 ){
-                RGB2HSV( r,g,b, &hue, &sat, &val );
-                alpha = (((float)val) / 100.0);
-                /* set pixel */
-                getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
-                tmp_a = (Uint8)( ((float)a) * alpha );
-            } else if( method == 1 ){
+    if( method == 0 ){
+        GETPIXELS_RGBA(surface_mask);
+            RGB2HSV( r,g,b, &hue, &sat, &val );
+            alpha = (((float)val) / 100.0f);
+            /* set pixel */
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
+            tmp_a = (Uint8)( ((float)a) * alpha );
+        SETPIXELS_RGBA(target_surface, tmp_a);
+    } else if( method == 1 ){
+        GETPIXELS_RGBA(surface_mask);
+            tmp_a = a;
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
+        SETPIXELS_RGBA(target_surface, tmp_a);
+    } else if( method == 2 ){
+        GETPIXELS_RGBA(surface_mask);
+            tmp_a = a;
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
+            /* alpha blend */
+            alpha = ((float) a) / 255.0f;
+            alpha2 = ((float) tmp_a) / 255.0f;
+            tmp_a = (Uint8) (alpha * alpha2) * 255;
+        SETPIXELS_RGBA(target_surface, tmp_a);
+    } else if( method == 3 ){
+        GETPIXELS_RGBA(surface_mask);
+            tmp_a = a;/*mask alpha*/
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
+            if(tmp_a > 0){/* if mask alpha > 0 then set original alpha */
                 tmp_a = a;
-                getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
-            } else if( method == 2 ){
-                tmp_a = a;
-                getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
-                /* alpha blend */
-                alpha = ((float) a) / 255.0;
-                alpha2 = ((float) tmp_a) / 255.0;
-                tmp_a = (alpha * alpha2) * 255;
-            } else if( method == 3 ){
-                tmp_a = a;/*mask alpha*/
-                getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
-                if(tmp_a > 0){/* if mask alpha > 0 then set original alpha */
-                    tmp_a = a;
-                }
-           } else {
-                tmp_a = 255;
             }
-
-            setPixel_RGBA(target_surface, x, y, r, g, b, tmp_a);
-
-        }
+        SETPIXELS_RGBA(target_surface, tmp_a);
+    } else {
+        GETPIXELS_RGBA(surface_mask);
+        SETPIXELS_RGBA(target_surface, 255);
     }
 
     return PySurface_New( target_surface );
@@ -645,6 +689,7 @@ static PyMethodDef ColorMethods[] = {
 
     {"brightness", brightness, METH_VARARGS, "Surface brightness"},
     {"contrast", contrast, METH_VARARGS, "Surface contrast"},
+    {"desaturate", desaturate, METH_VARARGS, "Desaturate surface"},
     {"invert", invert, METH_VARARGS, "Invert colors of surface"},
     {"greyscale", greyscale, METH_VARARGS, "Create greyscale surface"},
     {"colorize", colorize, METH_VARARGS, "Change hue of surface"},
