@@ -33,6 +33,7 @@
 */
 
 #include "pgext.h"
+#include "getset.h"
 
 static SDL_Surface*
 _shadow( SDL_Surface *surface, int radius, float opacity, int w, int h, int rw, int rh, int offset, Uint8 color[4] );
@@ -136,10 +137,10 @@ _blur( SDL_Surface *source, SDL_Surface *target, int radius, char vertical )
             if(color_cnt <= 1) continue;
 
             /* average channel value */
-            color[0] = sumcolor[0] / color_cnt;
-            color[1] = sumcolor[1] / color_cnt;
-            color[2] = sumcolor[2] / color_cnt;
-            color[3] = sumcolor[3] / color_cnt;
+            color[0] = (Uint8) (sumcolor[0] / color_cnt);
+            color[1] = (Uint8) (sumcolor[1] / color_cnt);
+            color[2] = (Uint8) (sumcolor[2] / color_cnt);
+            color[3] = (Uint8) (sumcolor[3] / color_cnt);
 
             /* map & set pixel */
             setPixel_RGBA(target, x, y, color[0], color[1], color[2], color[3]);
